@@ -15,8 +15,8 @@ def test_init(rng, samples):
 
 
 def test_init_kwargs(rng):
-    ss = [{'i': 3}, {'i': 1}, {'i': 2}, {'i': 5}, {'i': 4}]
-    iter_ = ShuffleIterator(ss, key=lambda s: s['i'], scale=2, rng=rng)
+    ss = [{"i": 3}, {"i": 1}, {"i": 2}, {"i": 5}, {"i": 4}]
+    iter_ = ShuffleIterator(ss, key=lambda s: s["i"], scale=2, rng=rng)
     assert_shuffled(ss, list(iter_))
 
 
@@ -26,14 +26,14 @@ def test_rng_called_correctly(rng, samples):
     list(ShuffleIterator(samples, rng=mock_rng))
     assert mock_rng.shuffle.call_count == 1
 
-    ss = [{'i': 3}, {'i': 1}, {'i': 2}, {'i': 5}, {'i': 4}]
-    list(ShuffleIterator(ss, key=lambda s: s['i'], rng=mock_rng))
+    ss = [{"i": 3}, {"i": 1}, {"i": 2}, {"i": 5}, {"i": 4}]
+    list(ShuffleIterator(ss, key=lambda s: s["i"], rng=mock_rng))
     assert mock_rng.uniform.call_count == len(ss)
 
 
 def test_init_zero_scale(rng):
-    ss = [{'i': 3}, {'i': 1}, {'i': 2}, {'i': 5}, {'i': 4}]
-    key = lambda s: s['i']
+    ss = [{"i": 3}, {"i": 1}, {"i": 2}, {"i": 5}, {"i": 4}]
+    key = lambda s: s["i"]
     iter_ = ShuffleIterator(ss, key=key, scale=0, rng=rng)
     assert sorted(ss, key=key) == list(iter_)
 
@@ -41,7 +41,7 @@ def test_init_zero_scale(rng):
 def test_init_negative_scale(rng, samples):
     with pytest.raises(ValueError) as exc:
         ShuffleIterator(samples, scale=-1)
-    assert 'scale cannot be less than 0' in str(exc.value)
+    assert "scale cannot be less than 0" in str(exc.value)
 
 
 def assert_shuffled(before, after):
