@@ -13,17 +13,18 @@
 # limitations under the License.
 """Convert your NLP text data to arrays!"""
 
-__version__ = '0.1.2'
+__version__ = "0.2.0"
 __all__ = [
-    'Sample',
-    'Batch',
-    'Vocab',
-    'StringStore',
-    'BatchIterator',
-    'ShuffleIterator',
+    "Sample",
+    "Batch",
+    "Vocab",
+    "StringStore",
+    "BatchIterator",
+    "BucketIterator",
+    "ShuffleIterator",
 ]
 
 from .batches import Batch
 from .samples import Sample
-from .iterators import BatchIterator, ShuffleIterator
+from .iterators import BatchIterator, BucketIterator, ShuffleIterator
 from .vocab import StringStore, Vocab
