@@ -12,11 +12,12 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+from collections import defaultdict
 from random import Random
 from typing import Any, Callable, Iterable, Iterator, Optional, Sequence, Sized
 import statistics as stat
 
-from text2array import Batch, Sample
+from . import Batch, Sample
 
 
 class BatchIterator(Iterable[Batch], Sized):
@@ -49,7 +50,7 @@ class BatchIterator(Iterable[Batch], Sized):
 
     def __init__(self, samples: Iterable[Sample], batch_size: int = 1) -> None:
         if batch_size <= 0:
-            raise ValueError('batch size must be greater than 0')
+            raise ValueError("batch size must be greater than 0")
 
         self._samples = samples
         self._bsz = batch_size
@@ -116,14 +117,14 @@ class ShuffleIterator(Iterable[Any], Sized):
     """
 
     def __init__(
-            self,
-            items: Sequence[Any],
-            key: Optional[Callable[[Any], int]] = None,
-            scale: float = 1.0,
-            rng: Optional[Random] = None,
+        self,
+        items: Sequence[Any],
+        key: Optional[Callable[[Any], int]] = None,
+        scale: float = 1.0,
+        rng: Optional[Random] = None,
     ) -> None:
         if scale < 0:
-            raise ValueError('scale cannot be less than 0')
+            raise ValueError("scale cannot be less than 0")
         if rng is None:  # pragma: no cover
             rng = Random()
 
@@ -158,3 +159,72 @@ class ShuffleIterator(Iterable[Any], Sized):
         shuf_items = [self._items[i] for i in indices]
 
         self._items = shuf_items
+
+
+class BucketIterator(Iterable[Batch], Sized):
+    """Iterator that batches together samples from the same bucket.
+
+    Example:
+
+        >>> from text2array import BucketIterator
+        >>> samples = [
+        ...   {'ws': ['a']},
+        ...   {'ws': ['a', 'b']},
+        ...   {'ws': ['b']},
+        ...   {'ws': ['c']},
+        ...   {'ws': ['b', 'b']},
+        ... ]
+        >>> iter_ = BucketIterator(samples, key=lambda s: len(s['ws']), batch_size=2)
+        >>> for b in iter_:
+        ...   print(list(b))
+        ...
+        [{'ws': ['a']}, {'ws': ['b']}]
+        [{'ws': ['c']}]
+        [{'ws': ['a', 'b']}, {'ws': ['b', 'b']}]
+
+    Args:
+        samples (~typing.Iterable[Sample]): Iterable of samples to batch.
+        key (typing.Callable[[Sample], Any]): Callable to get the bucket key of a sample.
+        batch_size: Maximum number of samples in each batch.
+        shuffle_bucket: Whether to shuffle every bucket before batching.
+        rng: Random number generator to use for shuffling. Set this to ensure reproducibility.
+            If not given, an instance of `~random.Random` with the default seed is used.
+
+    Note:
+        When ``samples`` is an instance of `~typing.Sized`, this iterator can
+        be passed to `len` to get the number of batches. Otherwise, a `TypeError`
+        is raised.
+    """
+
+    def __init__(
+        self,
+        samples: Iterable[Sample],
+        key: Callable[[Sample], Any],
+        batch_size: int = 1,
+        shuffle_bucket: bool = False,
+        rng: Optional[Random] = None,
+    ) -> None:
+        if rng is None:  # pragma: no cover
+            rng = Random()
+
+        self._bsz = batch_size
+        self._shuf = shuffle_bucket
+        self._rng = rng
+
+        bucket_dict = defaultdict(list)
+        for s in samples:
+            bucket_dict[key(s)].append(s)
+        self._buckets = bucket_dict.values()
+
+    @property
+    def batch_size(self):
+        return self._bsz
+
+    def __len__(self):
+        return sum(len(BatchIterator(ss, self._bsz)) for ss in self._buckets)
+
+    def __iter__(self):
+        for ss in self._buckets:
+            if self._shuf:
+                self._rng.shuffle(ss)
+            yield from BatchIterator(ss, self._bsz)
