@@ -13,21 +13,31 @@
 # limitations under the License.
 
 from collections import Counter, UserDict, defaultdict
-from typing import Counter as CounterT, Dict, Iterable, Iterator, Mapping, \
-    MutableMapping, Optional, Sequence, Set
+from typing import (
+    Counter as CounterT,
+    Dict,
+    Iterable,
+    Iterator,
+    Mapping,
+    MutableMapping,
+    Optional,
+    Sequence,
+    Set,
+)
 
-from ordered_set import OrderedSet
-from tqdm import tqdm
+from ordered_set import OrderedSet  # type: ignore
+from tqdm import tqdm  # type: ignore
 
 from .samples import FieldName, FieldValue, Sample
 
 
-class Vocab(UserDict, MutableMapping[FieldName, 'StringStore']):
+class Vocab(UserDict, MutableMapping[FieldName, "StringStore"]):
     """A dictionary from field names to `StringStore` objects as the field's vocabulary."""
-    PAD_TOKEN = '<pad>'
-    UNK_TOKEN = '<unk>'
 
-    def __getitem__(self, name: FieldName) -> 'StringStore':
+    PAD_TOKEN = "<pad>"
+    UNK_TOKEN = "<unk>"
+
+    def __getitem__(self, name: FieldName) -> "StringStore":
         try:
             return super().__getitem__(name)
         except KeyError:
@@ -64,11 +74,11 @@ class Vocab(UserDict, MutableMapping[FieldName, 'StringStore']):
 
     @classmethod
     def from_samples(
-            cls,
-            samples: Iterable[Sample],
-            options: Optional[Mapping[FieldName, dict]] = None,
-            pbar: Optional[tqdm] = None,
-    ) -> 'Vocab':
+        cls,
+        samples: Iterable[Sample],
+        options: Optional[Mapping[FieldName, dict]] = None,
+        pbar: Optional[tqdm] = None,
+    ) -> "Vocab":
         """Make an instance of this class from an iterable of samples.
 
         A vocabulary is only made for fields whose value is a string token or a (nested)
@@ -104,7 +114,7 @@ class Vocab(UserDict, MutableMapping[FieldName, 'StringStore']):
             Vocab: Vocabulary instance.
         """
         if pbar is None:  # pragma: no cover
-            pbar = tqdm(samples, desc='Counting', unit='sample')
+            pbar = tqdm(samples, desc="Counting", unit="sample")
         if options is None:
             options = {}
 
@@ -124,8 +134,8 @@ class Vocab(UserDict, MutableMapping[FieldName, 'StringStore']):
             opts = options.get(name, {})
 
             # Padding and unknown tokens
-            pad = opts.get('pad', cls.PAD_TOKEN)
-            unk = opts.get('unk', cls.UNK_TOKEN)
+            pad = opts.get("pad", cls.PAD_TOKEN)
+            unk = opts.get("unk", cls.UNK_TOKEN)
             inits = []
             if name in seqfield and pad is not None:
                 inits.append(pad)
@@ -134,8 +144,8 @@ class Vocab(UserDict, MutableMapping[FieldName, 'StringStore']):
 
             store = StringStore(inits, default=unk)
 
-            min_count = opts.get('min_count', 1)
-            max_size = opts.get('max_size')
+            min_count = opts.get("min_count", 1)
+            max_size = opts.get("max_size")
             n = len(store)
             for tok, freq in c.most_common():
                 if freq < min_count or (max_size is not None and len(store) - n >= max_size):
@@ -145,6 +155,27 @@ class Vocab(UserDict, MutableMapping[FieldName, 'StringStore']):
 
         return cls(m)
 
+    def extend(
+        self, samples: Iterable[Sample], fields: Optional[Iterable[FieldName]] = None,
+    ) -> None:
+        """Extend vocabulary with field values in samples.
+
+        Args:
+            samples (~typing.Iterable[Sample]): Samples to extend the vocabulary with.
+            fields: Extend only these field names. Defaults to all field names in the
+                vocabulary.
+        """
+        if fields is None:
+            fields = self.keys()
+
+        for s in samples:
+            for name in fields:
+                store = self[name]
+                val = s[name]
+                if isinstance(val, str):
+                    val = [val]
+                store.update(val)
+
     @classmethod
     def _needs_vocab(cls, val: FieldValue) -> bool:
         if isinstance(val, str):
@@ -176,7 +207,7 @@ class Vocab(UserDict, MutableMapping[FieldName, 'StringStore']):
         return s
 
     @classmethod
-    def _index_value(cls, store: 'StringStore', value: FieldValue) -> FieldValue:
+    def _index_value(cls, store: "StringStore", value: FieldValue) -> FieldValue:
         if isinstance(value, str):
             return store.index(value)
         if not isinstance(value, Sequence):
@@ -185,7 +216,7 @@ class Vocab(UserDict, MutableMapping[FieldName, 'StringStore']):
         return [cls._index_value(store, v) for v in value]
 
     @classmethod
-    def _get_value(cls, store: 'StringStore', value: FieldValue) -> FieldValue:
+    def _get_value(cls, store: "StringStore", value: FieldValue) -> FieldValue:
         if not isinstance(value, Sequence):
             return store[value]
         if isinstance(value, str):
@@ -224,10 +255,9 @@ class StringStore(OrderedSet):
         default: Default string as a representation of unknown strings, i.e. those that
             do not exist in the store.
     """
+
     def __init__(
-            self,
-            initial: Optional[Sequence[str]] = None,
-            default: Optional[str] = None,
+        self, initial: Optional[Sequence[str]] = None, default: Optional[str] = None,
     ) -> None:
         super().__init__(initial)
         self.default = default
@@ -246,4 +276,16 @@ class StringStore(OrderedSet):
         return self.default == o.default and super().__eq__(o)
 
     def __repr__(self) -> str:
-        return f'{self.__class__.__name__}({list(self)!r}, default={self.default!r})'
+        return f"{self.__class__.__name__}({list(self)!r}, default={self.default!r})"
+
+    # We need to customize __getstate__ and __setstate__ because OrderedSet override these
+    # to pickle only the sequence of items, resulting in `default` always set to None
+    def __getstate__(self):
+        return {
+            "initial": super().__getstate__(),
+            "default": self.default,
+        }
+
+    def __setstate__(self, state):
+        super().__setstate__(state.get("initial", []))
+        self.default = state.get("default")
